package qrz

import (
	"fmt"
	"io/ioutil"
	"net/http"
	"regexp"
	"strings"

	"git.paulbsd.com/paulbsd/qrz/src/config"
	"github.com/antchfx/htmlquery"
	"github.com/jmoiron/sqlx"
	"golang.org/x/text/encoding/charmap"
)

// GetBody fetch html page
func GetBody(config config.Config) (string, error) {
	var client http.Client

	resp, err := client.Get(config.FrsConfig.URL)
	if err != nil {
		return "", err
	}

	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return "", err
	}
	defer resp.Body.Close()

	dec := charmap.ISO8859_1.NewDecoder()
	output, _ := dec.Bytes(body)
	bodyString := string(output)

	return bodyString, nil
}

// GetFrsEntries get FRS entries from html body
func GetFrsEntries(config config.Config, body string) ([]FrsEntry, error) {
	var err error

	var frsentries []FrsEntry
	re := regexp.MustCompile(config.FrsConfig.Regex)

	htmlpage, err := htmlquery.Parse(strings.NewReader(body))
	if err != nil {
		return frsentries, err
	}

	for _, n := range htmlquery.Find(htmlpage, config.FrsConfig.XPath) {
		var frs FrsEntry
		td := htmlquery.Find(n, "//td")
		if re.MatchString(htmlquery.InnerText(td[0])) {
			frs.QRZ = strings.Replace(htmlquery.InnerText(td[0]), "'", "\\'", -1)
			frs.Name = strings.Replace(htmlquery.InnerText(td[1]), "'", "\\'", -1)
			frs.City = strings.Replace(htmlquery.InnerText(td[2]), "'", "\\'", -1)
			frs.Dept = strings.Replace(htmlquery.InnerText(td[3]), "'", "\\'", -1)
			frs.Country = strings.Replace(htmlquery.InnerText(td[4]), "'", "\\'", -1)

			frsentries = append(frsentries, frs)
		}
	}
	return frsentries, nil
}

// InsertFrsEntryToDB inserts frs entries to database
func InsertFrsEntryToDB(config config.Config, db sqlx.DB, frsPeople []FrsEntry) error {
	var err error
	tx := db.MustBegin()
	var qrzNum int

	fmt.Println(fmt.Sprintf("Starting insert %d entries", len(frsPeople)))

	for _, j := range frsPeople {
		query := fmt.Sprintf(config.DbInsertStatement, config.DbTable, j.QRZ, j.Name, j.City, j.Dept, j.Country)
		tx.MustExec(query)
		qrzNum++
	}

	err = tx.Commit()
	if err != nil {
		return err
	}
	fmt.Println(fmt.Sprintf("Commited %d entries", qrzNum))

	return nil
}

// FrsEntry describe FRS people
type FrsEntry struct {
	QRZ     string `db:"qrz"`
	Name    string `db:"name"`
	City    string `db:"city"`
	Dept    string `db:"dept"`
	Country string `db:"country"`
}
